/*
  Copyright (c) 2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once


#include "Event.hpp"

#include <deque>
#include <limits>
#include <memory>
#include <queue>


namespace RPGEdit {

namespace Control {

class EventBus {
private:
	typedef std::pair<uint64_t, std::shared_ptr<Event>> event_entry;

	static bool compare_events(const event_entry &e1, const event_entry &e2) {
		return e1.first > e2.first;
	}

	std::priority_queue<event_entry, std::deque<event_entry>, bool (*)(const event_entry &, const event_entry &)> events;

public:
	EventBus() : events(compare_events) {
	}

	void enqueue(const std::shared_ptr<Event> &event, uint64_t time) {
		events.push(std::pair<uint64_t, std::shared_ptr<Event>>(time, event));
	}

	std::pair<uint64_t, std::shared_ptr<Event>> get(uint64_t time) {
		if (events.empty())
			return std::make_pair<uint64_t, std::shared_ptr<Event>>(std::numeric_limits<uint64_t>::max(), nullptr);

		std::pair<uint64_t, std::shared_ptr<Event>> top = events.top();

		if (top.first <= time) {
			events.pop();
			return top;
		}
		else
			return std::pair<uint64_t, std::shared_ptr<Event>>(top.first, std::shared_ptr<Event>());
	}
};

}

}
